@@ -1,6 +1,5 @@
 /* 
-   Unix SMB/Netbios implementation.
-   Version 2.2.
+   Unix SMB/CIFS implementation.
    Samba system utilities for ACL support.
    Copyright (C) Jeremy Allison 2000.
    
@@ -159,7 +158,7 @@ int sys_acl_valid( SMB_ACL_T theacl )
 	return acl_valid(theacl);
 }
 
-int sys_acl_set_file( const char *name, SMB_ACL_TYPE_T acltype, SMB_ACL_T theacl)
+int sys_acl_set_file(const char *name, SMB_ACL_TYPE_T acltype, SMB_ACL_T theacl)
 {
 	return acl_set_file(name, acltype, theacl);
 }
@@ -645,13 +644,7 @@ char *sys_acl_to_text(SMB_ACL_T acl_d, ssize_t *len_p)
 				break;
 
 			case SMB_ACL_USER:
-				if ((pw = sys_getpwuid(ap->a_id)) == NULL) {
-					slprintf(idbuf, sizeof(idbuf)-1, "%ld",
-						(long)ap->a_id);
-					id = idbuf;
-				} else {
-					id = pw->pw_name;
-				}
+				id = uidtoname(ap->a_id);
 			case SMB_ACL_USER_OBJ:
 				tag = "user";
 				break;
@@ -1204,7 +1197,7 @@ SMB_ACL_T sys_acl_get_fd(int fd)
 	 * can use the relative path.
 	 */
 
-	return sys_acl_get_file(dos_to_unix_static(fsp->fsp_name), SMB_ACL_TYPE_ACCESS);
+	return sys_acl_get_file(fsp->fsp_name, SMB_ACL_TYPE_ACCESS);
 }
 
 int sys_acl_clear_perms(SMB_ACL_PERMSET_T permset_d)
@@ -1282,13 +1275,7 @@ char *sys_acl_to_text(SMB_ACL_T acl_d, ssize_t *len_p)
 				break;
 
 			case SMB_ACL_USER:
-				if ((pw = sys_getpwuid(ap->a_id)) == NULL) {
-					slprintf(idbuf, sizeof(idbuf)-1, "%ld",
-						(long)ap->a_id);
-					id = idbuf;
-				} else {
-					id = pw->pw_name;
-				}
+				id = uidtoname(ap->a_id);
 			case SMB_ACL_USER_OBJ:
 				tag = "user";
 				break;
@@ -1335,7 +1322,7 @@ char *sys_acl_to_text(SMB_ACL_T acl_d, ssize_t *len_p)
 			maxlen += nbytes + 20 * (acl_d->count - i);
 
 			if ((text = Realloc(oldtext, maxlen)) == NULL) {
-				SAFE_FREE(oldtext);
+				free(oldtext);
 				errno = ENOMEM;
 				return NULL;
 			}
@@ -1865,7 +1852,9 @@ int sys_acl_set_file(const char *name, SMB_ACL_TYPE_T type, SMB_ACL_T acl_d)
 
 	ret = acl(name, ACL_SET, acl_count, acl_p);
 
-	SAFE_FREE(acl_buf);
+	if (acl_buf) {
+		free(acl_buf);
+	}
 
 	return ret;
 }
@@ -1892,7 +1881,7 @@ int sys_acl_set_fd(int fd, SMB_ACL_T acl_d)
 	 * can use the relative path.
 	 */
 
-	return sys_acl_set_file(dos_to_unix_static(fsp->fsp_name), SMB_ACL_TYPE_ACCESS, acl_d);
+	return sys_acl_set_file(fsp->fsp_name, SMB_ACL_TYPE_ACCESS, acl_d);
 }
 
 int sys_acl_delete_def_file(const char *path)
@@ -1917,13 +1906,13 @@ int sys_acl_delete_def_file(const char *path)
 
 int sys_acl_free_text(char *text)
 {
-	SAFE_FREE(text);
+	free(text);
 	return 0;
 }
 
 int sys_acl_free_acl(SMB_ACL_T acl_d) 
 {
-	SAFE_FREE(acl_d);
+	free(acl_d);
 	return 0;
 }
 
@@ -2937,7 +2926,7 @@ int sys_acl_set_file( const char *name, SMB_ACL_TYPE_T acltype, SMB_ACL_T theacl
 		memcpy(acl_entry->ace_id->id_data, &user_id, sizeof(uid_t));
 	}
 
-	rc = chacl((char *)name,file_acl,file_acl->acl_len);
+	rc = chacl(name,file_acl,file_acl->acl_len);
 	DEBUG(10,("errno is %d\n",errno));
 	DEBUG(10,("return code is %d\n",rc));
 	SAFE_FREE(file_acl);
