@@ -80,16 +80,54 @@ static void nss_wins_init(void)
 	DEBUGLEVEL = 0;
 	AllowDebugChange = False;
 
-	/* needed for lp_xx() functions */
-	charset_initialise();
-
 	TimeInit();
 	setup_logging("nss_wins",False);
-	lp_load(CONFIGFILE,True,False,False);
+	lp_load(dyn_CONFIGFILE,True,False,False);
 	load_interfaces();
-	codepage_initialise(lp_client_code_page());
 }
 
+static struct in_addr *lookup_byname_backend(const char *name, int *count)
+{
+	int fd = -1;
+	struct ip_service *address = NULL;
+	struct in_addr *ret;
+	int j, flags = 0;
+
+	if (!initialised) {
+		nss_wins_init();
+	}
+
+	*count = 0;
+
+	/* always try with wins first */
+	if (resolve_wins(name,0x00,&address,count)) {
+		if ( (ret = (struct in_addr *)malloc(sizeof(struct in_addr))) == NULL ) {
+			free( address );
+			return NULL;
+		}
+		*ret = address[0].ip;
+		free( address );
+		return ret;
+	}
+
+	fd = wins_lookup_open_socket_in();
+	if (fd == -1) {
+		return NULL;
+	}
+
+	/* uggh, we have to broadcast to each interface in turn */
+	for (j=iface_count() - 1;j >= 0;j--) {
+		struct in_addr *bcast = iface_n_bcast(j);
+		ret = name_query(fd,name,0x00,True,True,*bcast,count, &flags, NULL);
+		if (ret) break;
+	}
+
+	close(fd);
+	return ret;
+}
+
+#ifdef HAVE_NS_API_H
+
 static struct node_status *lookup_byaddr_backend(char *addr, int *count)
 {
 	int fd;
@@ -107,58 +145,12 @@ static struct node_status *lookup_byaddr_backend(char *addr, int *count)
 
 	make_nmb_name(&nname, "*", 0);
 	ip = *interpret_addr2(addr);
-	status = node_status_query(fd,&nname,ip, count);
+	status = node_status_query(fd,&nname,ip, count, NULL);
 
 	close(fd);
 	return status;
 }
 
-static struct in_addr *lookup_byname_backend(const char *name, int *count)
-{
-	int fd;
-	struct in_addr *ret = NULL;
-	struct in_addr  p;
-	int j, flags;
-
-	if (!initialised) {
-		nss_wins_init();
-	}
-
-	*count = 0;
-
-	fd = wins_lookup_open_socket_in();
-	if (fd == -1)
-		return NULL;
-
-	p = wins_srv_ip();
-	if( !is_zero_ip(p) ) {
-		ret = name_query(fd,name,0x20,False,True, p, count, &flags);
-		goto out;
-	}
-
-	if (lp_wins_support()) {
-		/* we are our own WINS server */
-		ret = name_query(fd,name,0x20,False,True, *interpret_addr2("127.0.0.1"), count, &flags);
-		goto out;
-	}
-
-	/* uggh, we have to broadcast to each interface in turn */
-	for (j=iface_count() - 1;
-	     j >= 0;
-	     j--) {
-		struct in_addr *bcast = iface_n_bcast(j);
-		ret = name_query(fd,name,0x20,True,True,*bcast,count, &flags);
-		if (ret) break;
-	}
-
- out:
-
-	close(fd);
-	return ret;
-}
-
-
-#ifdef HAVE_NS_API_H
 /* IRIX version */
 
 int init(void)
@@ -202,7 +194,7 @@ int lookup(nsd_file_t *rq)
 	 * response needs to be a string of the following format
 	 * ip_address[ ip_address]*\tname[ alias]*
 	 */
-	if (strcasecmp(map,"hosts.byaddr") == 0) {
+	if (StrCaseCmp(map,"hosts.byaddr") == 0) {
 		if ( status = lookup_byaddr_backend(key, &count)) {
 		    size = strlen(key) + 1;
 		    if (size > len) {
@@ -230,7 +222,7 @@ int lookup(nsd_file_t *rq)
 		    response[strlen(response)-1] = '\n';
 		    free(status);
 		}
-	} else if (strcasecmp(map,"hosts.byname") == 0) {
+	} else if (StrCaseCmp(map,"hosts.byname") == 0) {
 	    if (ip_list = lookup_byname_backend(key, &count)) {
 		for (i = count; i ; i--) {
 		    addr = inet_ntoa(ip_list[i-1]);
@@ -268,72 +260,120 @@ int lookup(nsd_file_t *rq)
 }
 
 #else
+
+/* Allocate some space from the nss static buffer.  The buffer and buflen
+   are the pointers passed in by the C library to the _nss_*_*
+   functions. */
+
+static char *get_static(char **buffer, size_t *buflen, int len)
+{
+	char *result;
+
+	/* Error check.  We return false if things aren't set up right, or
+	   there isn't enough buffer space left. */
+	
+	if ((buffer == NULL) || (buflen == NULL) || (*buflen < len)) {
+		return NULL;
+	}
+
+	/* Return an index into the static buffer */
+
+	result = *buffer;
+	*buffer += len;
+	*buflen -= len;
+
+	return result;
+}
+
 /****************************************************************************
 gethostbyname() - we ignore any domain portion of the name and only
 handle names that are at most 15 characters long
   **************************************************************************/
 NSS_STATUS
-_nss_wins_gethostbyname_r(const char *name, struct hostent *he,
-			  char *buffer, size_t buflen, int *errnop,
-			  int *h_errnop)
+_nss_wins_gethostbyname_r(const char *hostname, struct hostent *he,
+			  char *buffer, size_t buflen, int *h_errnop)
 {
-	char **host_addresses;
 	struct in_addr *ip_list;
 	int i, count;
-	size_t namelen = strlen(name) + 1;
+	fstring name;
+	size_t namelen;
 		
 	memset(he, '\0', sizeof(*he));
+	fstrcpy(name, hostname);
+
+	/* Do lookup */
 
 	ip_list = lookup_byname_backend(name, &count);
-	if (!ip_list) {
+
+	if (!ip_list)
 		return NSS_STATUS_NOTFOUND;
+
+	/* Copy h_name */
+
+	namelen = strlen(name) + 1;
+
+	if ((he->h_name = get_static(&buffer, &buflen, namelen)) == NULL)
+		return NSS_STATUS_TRYAGAIN;
+
+	memcpy(he->h_name, name, namelen);
+
+	/* Copy h_addr_list, align to pointer boundary first */
+
+	if ((i = (unsigned long)(buffer) % sizeof(char*)) != 0)
+		i = sizeof(char*) - i;
+
+	if (get_static(&buffer, &buflen, i) == NULL)
+		return NSS_STATUS_TRYAGAIN;
+
+	if ((he->h_addr_list = (char **)get_static(
+		     &buffer, &buflen, (count + 1) * sizeof(char *))) == NULL)
+		return NSS_STATUS_TRYAGAIN;
+
+	for (i = 0; i < count; i++) {
+		if ((he->h_addr_list[i] = get_static(&buffer, &buflen,
+						     INADDRSZ)) == NULL)
+			return NSS_STATUS_TRYAGAIN;
+		memcpy(he->h_addr_list[i], &ip_list[i], INADDRSZ);
 	}
 
-	if (buflen < namelen + (2*count+1)*INADDRSZ) {
-		/* no ENOMEM error type?! */
-		return NSS_STATUS_NOTFOUND;
-	}
-
-
-	host_addresses = (char **)buffer;
-	he->h_addr_list = host_addresses;
-	host_addresses[count] = NULL;
-	buffer += (count + 1) * INADDRSZ;
-	buflen += (count + 1) * INADDRSZ;
-	he->h_addrtype = AF_INET;
-	he->h_length = INADDRSZ;
-
-	for (i=0;i<count;i++) {
-		memcpy(buffer, &ip_list[i].s_addr, INADDRSZ);
-		*host_addresses = buffer;
-		buffer += INADDRSZ;
-		buflen -= INADDRSZ;
-		host_addresses++;
-	}
+	he->h_addr_list[count] = NULL;
 
 	if (ip_list)
 		free(ip_list);
 
-	memcpy(buffer, name, namelen);
-	he->h_name = buffer;
+	/* Set h_addr_type and h_length */
+
+	he->h_addrtype = AF_INET;
+	he->h_length = INADDRSZ;
+
+	/* Set h_aliases */
+
+	if ((i = (unsigned long)(buffer) % sizeof(char*)) != 0)
+		i = sizeof(char*) - i;
+
+	if (get_static(&buffer, &buflen, i) == NULL)
+		return NSS_STATUS_TRYAGAIN;
+
+	if ((he->h_aliases = (char **)get_static(
+		     &buffer, &buflen, sizeof(char *))) == NULL)
+		return NSS_STATUS_TRYAGAIN;
+
+	he->h_aliases[0] = NULL;
 
 	return NSS_STATUS_SUCCESS;
 }
 
+
 NSS_STATUS
 _nss_wins_gethostbyname2_r(const char *name, int af, struct hostent *he,
-				char *buffer, size_t buflen, int *errnop,
-				int *h_errnop)
+			   char *buffer, size_t buflen, int *h_errnop)
 {
 	if(af!=AF_INET) {
 		*h_errnop = NO_DATA;
-		*errnop = EAFNOSUPPORT;
 		return NSS_STATUS_UNAVAIL;
 	}
 
-	return _nss_wins_gethostbyname_r(name,he,buffer,buflen,errnop,h_errnop);
+	return _nss_wins_gethostbyname_r(
+		name, he, buffer, buflen, h_errnop);
 }
-
-
-
 #endif
