@@ -191,31 +191,17 @@ static int EatComment( myFILE *InFile )
  *
  *****************************************************************************/
 
-static int Continuation( char *line, int pos )
+static int Continuation(char *line, int pos )
 {
-  int pos2 = 0;
+	pos--;
+	while( (pos >= 0) && isspace((int)line[pos]))
+		pos--;
 
-  pos--;
-  while( (pos >= 0) && isspace((int)line[pos]) )
-     pos--;
-
-  /* we should recognize if `\` is part of a multibyte character or not. */
-  while(pos2 <= pos) {
-    size_t skip = 0;
-    skip = get_character_len(line[pos2]);
-    if (skip) {
-        pos2 += skip;
-    } else if (pos == pos2) {
-        return( ((pos >= 0) && ('\\' == line[pos])) ? pos : -1 );
-    } else  {
-        pos2++;
-    }
-  }
-  return (-1);
+	return (((pos >= 0) && ('\\' == line[pos])) ? pos : -1 );
 }
 
 
-static BOOL Section( myFILE *InFile, BOOL (*sfunc)(char *) )
+static BOOL Section( myFILE *InFile, BOOL (*sfunc)(const char *) )
   /* ------------------------------------------------------------------------ **
    * Scan a section name, and pass the name to function sfunc().
    *
@@ -233,7 +219,7 @@ static BOOL Section( myFILE *InFile, BOOL (*sfunc)(char *) )
   int   c;
   int   i;
   int   end;
-  char *func  = "params.c:Section() -";
+  const char *func  = "params.c:Section() -";
 
   i = 0;      /* <i> is the offset of the next free byte in bufr[] and  */
   end = 0;    /* <end> is the current "end of string" offset.  In most  */
@@ -251,7 +237,7 @@ static BOOL Section( myFILE *InFile, BOOL (*sfunc)(char *) )
     if( i > (bSize - 2) )
       {
       char *tb;
-
+      
       tb = Realloc( bufr, bSize +BUFR_INC );
       if( NULL == tb )
         {
@@ -272,7 +258,7 @@ static BOOL Section( myFILE *InFile, BOOL (*sfunc)(char *) )
           DEBUG(0, ("%s Empty section name in configuration file.\n", func ));
           return( False );
           }
-        if( !sfunc( unix_to_dos(bufr) ) )            /* Got a valid name.  Deal with it. */
+        if( !sfunc(bufr) )            /* Got a valid name.  Deal with it. */
           return( False );
         (void)EatComment( InFile );     /* Finish off the line.             */
         return( True );
@@ -311,7 +297,7 @@ static BOOL Section( myFILE *InFile, BOOL (*sfunc)(char *) )
   return( False );
   } /* Section */
 
-static BOOL Parameter( myFILE *InFile, BOOL (*pfunc)(char *, char *), int c )
+static BOOL Parameter( myFILE *InFile, BOOL (*pfunc)(const char *, const char *), int c )
   /* ------------------------------------------------------------------------ **
    * Scan a parameter name and value, and pass these two fields to pfunc().
    *
@@ -339,7 +325,7 @@ static BOOL Parameter( myFILE *InFile, BOOL (*pfunc)(char *, char *), int c )
   int   i       = 0;    /* Position within bufr. */
   int   end     = 0;    /* bufr[end] is current end-of-string. */
   int   vstart  = 0;    /* Starting position of the parameter value. */
-  char *func    = "params.c:Parameter() -";
+  const char *func    = "params.c:Parameter() -";
 
   /* Read the parameter name. */
   while( 0 == vstart )  /* Loop until we've found the start of the value. */
@@ -348,7 +334,7 @@ static BOOL Parameter( myFILE *InFile, BOOL (*pfunc)(char *, char *), int c )
     if( i > (bSize - 2) )       /* Ensure there's space for next char.    */
       {
       char *tb;
- 
+      
       tb = Realloc( bufr, bSize + BUFR_INC );
       if( NULL == tb )
         {
@@ -415,13 +401,16 @@ static BOOL Parameter( myFILE *InFile, BOOL (*pfunc)(char *, char *), int c )
 
     if( i > (bSize - 2) )       /* Make sure there's enough room. */
       {
-      bSize += BUFR_INC;
-      bufr   = Realloc( bufr, bSize );
-      if( NULL == bufr )
+      char *tb;
+      
+      tb = Realloc( bufr, bSize + BUFR_INC );
+      if( NULL == tb )
         {
         DEBUG(0, ("%s Memory re-allocation failure.", func) );
         return( False );
         }
+      bufr = tb;
+      bSize += BUFR_INC;
       }
 
     switch( c )
@@ -456,8 +445,8 @@ static BOOL Parameter( myFILE *InFile, BOOL (*pfunc)(char *, char *), int c )
   } /* Parameter */
 
 static BOOL Parse( myFILE *InFile,
-                   BOOL (*sfunc)(char *),
-                   BOOL (*pfunc)(char *, char *) )
+                   BOOL (*sfunc)(const char *),
+                   BOOL (*pfunc)(const char *, const char *) )
   /* ------------------------------------------------------------------------ **
    * Scan & parse the input.
    *
@@ -516,7 +505,7 @@ static BOOL Parse( myFILE *InFile,
   return( True );
   } /* Parse */
 
-static myFILE *OpenConfFile( char *FileName )
+static myFILE *OpenConfFile( const char *FileName )
   /* ------------------------------------------------------------------------ **
    * Open a configuration file.
    *
@@ -527,7 +516,7 @@ static myFILE *OpenConfFile( char *FileName )
    * ------------------------------------------------------------------------ **
    */
   {
-  char *func = "params.c:OpenConfFile() -";
+  const char *func = "params.c:OpenConfFile() -";
   extern BOOL in_client;
   int lvl = in_client?1:0;
   myFILE *ret;
@@ -549,9 +538,9 @@ static myFILE *OpenConfFile( char *FileName )
   return( ret );
   } /* OpenConfFile */
 
-BOOL pm_process( char *FileName,
-                 BOOL (*sfunc)(char *),
-                 BOOL (*pfunc)(char *, char *) )
+BOOL pm_process( const char *FileName,
+                 BOOL (*sfunc)(const char *),
+                 BOOL (*pfunc)(const char *, const char *) )
   /* ------------------------------------------------------------------------ **
    * Process the named parameter file.
    *
@@ -568,7 +557,7 @@ BOOL pm_process( char *FileName,
   {
   int   result;
   myFILE *InFile;
-  char *func = "params.c:pm_process() -";
+  const char *func = "params.c:pm_process() -";
 
   InFile = OpenConfFile( FileName );          /* Open the config file. */
   if( NULL == InFile )
