@@ -1,6 +1,5 @@
 /* 
-   Unix SMB/Netbios implementation.
-   Version 1.9.
+   Unix SMB/CIFS implementation.
    NT Domain Authentication SMB / MSRPC client
    Copyright (C) Andrew Tridgell 1994-1997
    Copyright (C) Luke Kenneth Casson Leighton 1996-1997
@@ -89,7 +88,7 @@ static void cmd_reg_enum(struct client_info *info)
 	}
 
 	/* open WINREG session. */
-	res = res ? cli_nt_session_open(smb_cli, PIPE_WINREG) : False;
+	res = res ? cli_nt_session_open(smb_cli, PI_WINREG) : False;
 
 	/* open registry receive a policy handle */
 	res = res ? do_reg_connect(smb_cli, full_keyname, key_name,
@@ -248,7 +247,7 @@ static void cmd_reg_query_key(struct client_info *info)
 	}
 
 	/* open WINREG session. */
-	res = res ? cli_nt_session_open(smb_cli, PIPE_WINREG) : False;
+	res = res ? cli_nt_session_open(smb_cli, PI_WINREG) : False;
 
 	/* open registry receive a policy handle */
 	res = res ? do_reg_connect(smb_cli, full_keyname, key_name,
@@ -411,7 +410,7 @@ static void cmd_reg_create_val(struct client_info *info)
 	dump_data(10, (char *)value.buffer, value.buf_len);
 
 	/* open WINREG session. */
-	res = res ? cli_nt_session_open(smb_cli, PIPE_WINREG) : False;
+	res = res ? cli_nt_session_open(smb_cli, PI_WINREG) : False;
 
 	/* open registry receive a policy handle */
 	res = res ? do_reg_connect(smb_cli, keyname, parent_name,
@@ -490,7 +489,7 @@ static void cmd_reg_delete_val(struct client_info *info)
 	}
 	
 	/* open WINREG session. */
-	res = res ? cli_nt_session_open(smb_cli, PIPE_WINREG) : False;
+	res = res ? cli_nt_session_open(smb_cli, PI_WINREG) : False;
 
 	/* open registry receive a policy handle */
 	res = res ? do_reg_connect(smb_cli, keyname, parent_name,
@@ -565,7 +564,7 @@ static void cmd_reg_delete_key(struct client_info *info)
 	}
 	
 	/* open WINREG session. */
-	res = res ? cli_nt_session_open(smb_cli, PIPE_WINREG) : False;
+	res = res ? cli_nt_session_open(smb_cli, PI_WINREG) : False;
 
 	/* open registry receive a policy handle */
 	res = res ? do_reg_connect(smb_cli, parent_name, key_name,
@@ -654,7 +653,7 @@ static void cmd_reg_create_key(struct client_info *info)
 	sam_access.mask = SEC_RIGHTS_READ;
 
 	/* open WINREG session. */
-	res = res ? cli_nt_session_open(smb_cli, PIPE_WINREG) : False;
+	res = res ? cli_nt_session_open(smb_cli, PI_WINREG) : False;
 
 	/* open registry receive a policy handle */
 	res = res ? do_reg_connect(smb_cli, parent_key, parent_name,
@@ -733,7 +732,7 @@ static void cmd_reg_test_key_sec(struct client_info *info)
 	}
 
 	/* open WINREG session. */
-	res = res ? cli_nt_session_open(smb_cli, PIPE_WINREG) : False;
+	res = res ? cli_nt_session_open(smb_cli, PI_WINREG) : False;
 
 	/* open registry receive a policy handle */
 	res = res ? do_reg_connect(smb_cli, full_keyname, key_name,
@@ -828,7 +827,7 @@ static void cmd_reg_get_key_sec(struct client_info *info)
 	}
 
 	/* open WINREG session. */
-	res = res ? cli_nt_session_open(smb_cli, PIPE_WINREG) : False;
+	res = res ? cli_nt_session_open(smb_cli, PI_WINREG) : False;
 
 	/* open registry receive a policy handle */
 	res = res ? do_reg_connect(smb_cli, full_keyname, key_name,
@@ -896,14 +895,13 @@ static void cmd_reg_get_key_sec(struct client_info *info)
 nt registry shutdown
 ****************************************************************************/
 static NTSTATUS cmd_reg_shutdown(struct cli_state *cli, TALLOC_CTX *mem_ctx,
-                                 int argc, char **argv)
+                                 int argc, const char **argv)
 {
-	extern char *optarg;
-	extern int optind;
 	NTSTATUS result = NT_STATUS_UNSUCCESSFUL;
 	fstring msg;
 	uint32 timeout = 20;
-	uint16 flgs = 0;
+	BOOL force = False;
+	BOOL reboot = False;
 	int opt;
 
 	*msg = 0;
@@ -911,37 +909,33 @@ static NTSTATUS cmd_reg_shutdown(struct cli_state *cli, TALLOC_CTX *mem_ctx,
 
 	while ((opt = getopt(argc, argv, "m:t:rf")) != EOF)
 	{
-		fprintf (stderr, "[%s]\n", argv[argc-1]);
+		/*fprintf (stderr, "[%s]\n", argv[argc-1]);*/
 	
 		switch (opt)
 		{
 			case 'm':
-			{
-				safe_strcpy(msg, optarg, sizeof(msg)-1);
-				fprintf (stderr, "[%s|%s]\n", optarg, msg);
+				fstrcpy(msg, optarg);
+				/*fprintf (stderr, "[%s|%s]\n", optarg, msg);*/
 				break;
-			}
+
 			case 't':
-			{
 				timeout = atoi(optarg);
-				fprintf (stderr, "[%s|%d]\n", optarg, timeout);
-			break;
-			}
-			case 'r':
-			{
-				flgs |= 0x100;
-			break;
-			}
-			case 'f':
-			{
-				flgs |= 0x001;
+				/*fprintf (stderr, "[%s|%d]\n", optarg, timeout);*/
 				break;
-			}
+
+			case 'r':
+				reboot = True;
+				break;
+
+			case 'f':
+				force = True;
+				break;
+
 		}
 	}
 
 	/* create an entry */
-	result = cli_reg_shutdown(cli, mem_ctx, msg, timeout, flgs);
+	result = cli_reg_shutdown(cli, mem_ctx, msg, timeout, reboot, force);
 
 	if (NT_STATUS_IS_OK(result))
 		DEBUG(5,("cmd_reg_shutdown: query succeeded\n"));
@@ -956,7 +950,7 @@ abort a shutdown
 ****************************************************************************/
 static NTSTATUS cmd_reg_abort_shutdown(struct cli_state *cli, 
                                        TALLOC_CTX *mem_ctx, int argc, 
-                                       char **argv)
+                                       const char **argv)
 {
 	NTSTATUS result = NT_STATUS_UNSUCCESSFUL;
 
@@ -976,11 +970,11 @@ struct cmd_set reg_commands[] = {
 
 	{ "REG"  },
 
-	{ "shutdown",		cmd_reg_shutdown,		PIPE_WINREG, "Remote Shutdown",
-				"[-m message] [-t timeout] [-r] [-f] (-r == reboot, -f == force)" },
+	{ "shutdown", RPC_RTYPE_NTSTATUS, cmd_reg_shutdown, NULL, PI_WINREG, "Remote Shutdown",
+				"syntax: shutdown [-m message] [-t timeout] [-r] [-h] [-f] (-r == reboot, -h == halt, -f == force)" },
 				
-	{ "abortshutdown",	cmd_reg_abort_shutdown,		PIPE_WINREG, "Abort Shutdown",
-				"" },				
+	{ "abortshutdown", RPC_RTYPE_NTSTATUS, cmd_reg_abort_shutdown, NULL, PI_WINREG, "Abort Shutdown",
+				"syntax: abortshutdown" },
 /*
 	{ "regenum",		cmd_reg_enum,			"Registry Enumeration",
 				"<keyname>" },
