@@ -1,8 +1,8 @@
 /* 
-   Unix SMB/Netbios implementation.
-   Version 1.9.
+   Unix SMB/CIFS implementation.
    Samba utility functions
    Copyright (C) Andrew Tridgell 1992-1998
+   Copyright (C) Andrew Bartlett 2001-2004
    
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -19,6 +19,8 @@
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
 
+/* These comments regard the code to change the user's unix password: */
+
 /* fork a child process to exec passwd and write to its
  * tty to change a users password. This is running as the
  * user who is attempting to change the password.
@@ -30,9 +32,6 @@
  * was included as a client to change passwords using the 'passwd' program
  * on the remote machine.
  *
- * This routine is called by set_user_password() in password.c only if ALLOW_PASSWORD_CHANGE
- * is defined in the compiler directives located in the Makefile.
- *
  * This code has been hacked by Bob Nance (nance@niehs.nih.gov) and Evan Patterson
  * (patters2@niehs.nih.gov) at the National Institute of Environmental Health Sciences
  * and rights to modify, distribute or incorporate this change to the CAP suite or
@@ -51,6 +50,14 @@
 
 extern struct passdb_ops pdb_ops;
 
+static NTSTATUS check_oem_password(const char *user,
+				   uchar password_encrypted_with_lm_hash[516], 
+				   const uchar old_lm_hash_encrypted[16],
+				   uchar password_encrypted_with_nt_hash[516], 
+				   const uchar old_nt_hash_encrypted[16],
+				   SAM_ACCOUNT **hnd, char *new_passwd,
+				   int new_passwd_size);
+
 #if ALLOW_CHANGE_PASSWORD
 
 static int findpty(char **slave)
@@ -58,7 +65,7 @@ static int findpty(char **slave)
 	int master;
 	static fstring line;
 	DIR *dirp;
-	char *dpname;
+	const char *dpname;
 
 #if defined(HAVE_GRANTPT)
 	/* Try to open /dev/ptmx. If that fails, fall through to old method. */
@@ -112,20 +119,18 @@ static int findpty(char **slave)
 	return (-1);
 }
 
-static int dochild(int master, char *slavedev, char *name,
-		   char *passwordprogram, BOOL as_root)
+static int dochild(int master, const char *slavedev, const struct passwd *pass,
+		   const char *passwordprogram, BOOL as_root)
 {
 	int slave;
 	struct termios stermios;
-	struct passwd *pass = Get_Pwnam(name, True);
 	gid_t gid;
 	uid_t uid;
 
 	if (pass == NULL)
 	{
 		DEBUG(0,
-		      ("dochild: user name %s doesn't exist in the UNIX password database.\n",
-		       name));
+		      ("dochild: user doesn't exist in the UNIX password database.\n"));
 		return False;
 	}
 
@@ -164,17 +169,17 @@ static int dochild(int master, char *slavedev, char *name,
 
 	/* Make slave stdin/out/err of child. */
 
-	if (dup2(slave, STDIN_FILENO) != STDIN_FILENO)
+	if (sys_dup2(slave, STDIN_FILENO) != STDIN_FILENO)
 	{
 		DEBUG(3, ("Could not re-direct stdin\n"));
 		return (False);
 	}
-	if (dup2(slave, STDOUT_FILENO) != STDOUT_FILENO)
+	if (sys_dup2(slave, STDOUT_FILENO) != STDOUT_FILENO)
 	{
 		DEBUG(3, ("Could not re-direct stdout\n"));
 		return (False);
 	}
-	if (dup2(slave, STDERR_FILENO) != STDERR_FILENO)
+	if (sys_dup2(slave, STDERR_FILENO) != STDERR_FILENO)
 	{
 		DEBUG(3, ("Could not re-direct stderr\n"));
 		return (False);
@@ -196,13 +201,15 @@ static int dochild(int master, char *slavedev, char *name,
 #ifdef ONLCR
  	stermios.c_oflag &= ~(ONLCR);
 #endif
-	if (tcsetattr(0, TCSANOW, &stermios) < 0) {
+	if (tcsetattr(0, TCSANOW, &stermios) < 0)
+	{
 		DEBUG(3, ("could not set attributes of pty\n"));
 		return (False);
 	}
 
 	/* make us completely into the right uid */
-	if (!as_root) {
+	if (!as_root)
+	{
 		become_user_permanently(uid, gid);
 	}
 
@@ -239,13 +246,14 @@ static int expect(int master, char *issue, char *expected)
 		if (strequal(expected, "."))
 			return True;
 
-		timeout = 2000;
+		/* Initial timeout. */
+		timeout = lp_passwd_chat_timeout() * 1000;
 		nread = 0;
 		buffer[nread] = 0;
 
-		while ((len = read_with_timeout(master, buffer + nread, 1,
-						sizeof(buffer) - nread - 1,
-						timeout)) > 0) {
+		while ((len = read_socket_with_timeout(master, buffer + nread, 1,
+						       sizeof(buffer) - nread - 1,
+						       timeout)) > 0) {
 			nread += len;
 			buffer[nread] = 0;
 
@@ -253,10 +261,12 @@ static int expect(int master, char *issue, char *expected)
 				/* Eat leading/trailing whitespace before match. */
 				pstring str;
 				pstrcpy( str, buffer);
-				trim_string( str, " ", " ");
+				trim_char( str, ' ', ' ');
 
-				if ((match = (unix_wild_match(expected, str) == 0)))
-					timeout = 200;
+				if ((match = (unix_wild_match(expected, str) == 0))) {
+					/* Now data has started to return, lower timeout. */
+					timeout = lp_passwd_chat_timeout() * 100;
+				}
 			}
 		}
 
@@ -318,7 +328,7 @@ static int talktochild(int master, const char *seq)
 	return (count > 0);
 }
 
-static BOOL chat_with_program(char *passwordprogram, char *name,
+static BOOL chat_with_program(char *passwordprogram, struct passwd *pass,
 			      char *chatsequence, BOOL as_root)
 {
 	char *slavedev;
@@ -327,12 +337,14 @@ static BOOL chat_with_program(char *passwordprogram, char *name,
 	int wstat;
 	BOOL chstat = False;
 
+	if (pass == NULL) {
+		DEBUG(0, ("chat_with_program: user doesn't exist in the UNIX password database.\n"));
+		return False;
+	}
+
 	/* allocate a pseudo-terminal device */
-	if ((master = findpty(&slavedev)) < 0)
-	{
-		DEBUG(3,
-		      ("Cannot Allocate pty for password change: %s\n",
-		       name));
+	if ((master = findpty(&slavedev)) < 0) {
+		DEBUG(3, ("chat_with_program: Cannot Allocate pty for password change: %s\n", pass->pw_name));
 		return (False);
 	}
 
@@ -343,40 +355,30 @@ static BOOL chat_with_program(char *passwordprogram, char *name,
 
 	CatchChildLeaveStatus();
 
-	if ((pid = sys_fork()) < 0)
-	{
-		DEBUG(3,
-		      ("Cannot fork() child for password change: %s\n",
-		       name));
+	if ((pid = sys_fork()) < 0) {
+		DEBUG(3, ("chat_with_program: Cannot fork() child for password change: %s\n", pass->pw_name));
 		close(master);
 		CatchChild();
 		return (False);
 	}
 
 	/* we now have a pty */
-	if (pid > 0)
-	{			/* This is the parent process */
-		if ((chstat = talktochild(master, chatsequence)) == False)
-		{
-			DEBUG(3,
-			      ("Child failed to change password: %s\n",
-			       name));
+	if (pid > 0) {			/* This is the parent process */
+		if ((chstat = talktochild(master, chatsequence)) == False) {
+			DEBUG(3, ("chat_with_program: Child failed to change password: %s\n", pass->pw_name));
 			kill(pid, SIGKILL);	/* be sure to end this process */
 		}
 
-		while ((wpid = sys_waitpid(pid, &wstat, 0)) < 0)
-		{
-			if (errno == EINTR)
-			{
+		while ((wpid = sys_waitpid(pid, &wstat, 0)) < 0) {
+			if (errno == EINTR) {
 				errno = 0;
 				continue;
 			}
 			break;
 		}
 
-		if (wpid < 0)
-		{
-			DEBUG(3, ("The process is no longer waiting!\n\n"));
+		if (wpid < 0) {
+			DEBUG(3, ("chat_with_program: The process is no longer waiting!\n\n"));
 			close(master);
 			CatchChild();
 			return (False);
@@ -389,29 +391,23 @@ static BOOL chat_with_program(char *passwordprogram, char *name,
 
 		close(master);
 
-		if (pid != wpid)
-		{
-			DEBUG(3,
-			      ("We were waiting for the wrong process ID\n"));
+		if (pid != wpid) {
+			DEBUG(3, ("chat_with_program: We were waiting for the wrong process ID\n"));
 			return (False);
 		}
-		if (WIFEXITED(wstat) == 0)
-		{
-			DEBUG(3,
-			      ("The process exited while we were waiting\n"));
+		if (WIFEXITED(wstat) && (WEXITSTATUS(wstat) != 0)) {
+			DEBUG(3, ("chat_with_program: The process exited with status %d \
+while we were waiting\n", WEXITSTATUS(wstat)));
 			return (False);
 		}
-		if (WEXITSTATUS(wstat) != 0)
-		{
-			DEBUG(3,
-			      ("The status of the process exiting was %d\n",
-			       wstat));
+#if defined(WIFSIGNALLED) && defined(WTERMSIG)
+		else if (WIFSIGNALLED(wstat)) {
+                        DEBUG(3, ("chat_with_program: The process was killed by signal %d \
+while we were waiting\n", WTERMSIG(wstat)));
 			return (False);
 		}
-
-	}
-	else
-	{
+#endif
+	} else {
 		/* CHILD */
 
 		/*
@@ -425,12 +421,9 @@ static BOOL chat_with_program(char *passwordprogram, char *name,
 		if (as_root)
 			become_root();
 
-		DEBUG(3,
-		      ("Dochild for user %s (uid=%d,gid=%d)\n", name,
-		       (int)getuid(), (int)getgid()));
-		chstat =
-			dochild(master, slavedev, name, passwordprogram,
-				as_root);
+		DEBUG(3, ("chat_with_program: Dochild for user %s (uid=%d,gid=%d) (as_root = %s)\n", pass->pw_name,
+		       (int)getuid(), (int)getgid(), BOOLSTR(as_root) ));
+		chstat = dochild(master, slavedev, pass, passwordprogram, as_root);
 
 		if (as_root)
 			unbecome_root();
@@ -439,47 +432,40 @@ static BOOL chat_with_program(char *passwordprogram, char *name,
 		 * The child should never return from dochild() ....
 		 */
 
-		DEBUG(0,
-		      ("chat_with_program: Error: dochild() returned %d\n",
-		       chstat));
+		DEBUG(0, ("chat_with_program: Error: dochild() returned %d\n", chstat));
 		exit(1);
 	}
 
 	if (chstat)
-		DEBUG(3,
-		      ("Password change %ssuccessful for user %s\n",
-		       (chstat ? "" : "un"), name));
+		DEBUG(3, ("chat_with_program: Password change %ssuccessful for user %s\n",
+		       (chstat ? "" : "un"), pass->pw_name));
 	return (chstat);
 }
 
-
-BOOL chgpasswd(char *name, const char *oldpass, const char *newpass, BOOL as_root)
+BOOL chgpasswd(const char *name, const struct passwd *pass, 
+	       const char *oldpass, const char *newpass, BOOL as_root)
 {
 	pstring passwordprogram;
 	pstring chatsequence;
 	size_t i;
 	size_t len;
 
-	strlower(name);
-	DEBUG(3, ("Password change for user: %s\n", name));
+	if (!oldpass) {
+		oldpass = "";
+	}
+
+	DEBUG(3, ("chgpasswd: Password change (as_root=%s) for user: %s\n", BOOLSTR(as_root), name));
 
 #if DEBUG_PASSWORD
-	DEBUG(100, ("Passwords: old=%s new=%s\n", oldpass, newpass));
+	DEBUG(100, ("chgpasswd: Passwords: old=%s new=%s\n", oldpass, newpass));
 #endif
 
 	/* Take the passed information and test it for minimum criteria */
-	/* Minimum password length */
-	if (strlen(newpass) < lp_min_passwd_length()) {
-		/* too short, must be at least MINPASSWDLENGTH */
-		DEBUG(0, ("Password Change: user %s, New password is shorter than minimum password length = %d\n",
-		       name, lp_min_passwd_length()));
-		return (False);	/* inform the user */
-	}
 
 	/* Password is same as old password */
 	if (strcmp(oldpass, newpass) == 0) {
 		/* don't allow same password */
-		DEBUG(2, ("Password Change: %s, New password is same as old\n", name));	/* log the attempt */
+		DEBUG(2, ("chgpasswd: Password Change: %s, New password is same as old\n", name));	/* log the attempt */
 		return (False);	/* inform the user */
 	}
 
@@ -491,8 +477,7 @@ BOOL chgpasswd(char *name, const char *oldpass, const char *newpass, BOOL as_roo
 	len = strlen(oldpass);
 	for (i = 0; i < len; i++) {
 		if (iscntrl((int)oldpass[i])) {
-			DEBUG(0,
-			      ("chat_with_program: oldpass contains control characters (disallowed).\n"));
+			DEBUG(0, ("chgpasswd: oldpass contains control characters (disallowed).\n"));
 			return False;
 		}
 	}
@@ -500,12 +485,11 @@ BOOL chgpasswd(char *name, const char *oldpass, const char *newpass, BOOL as_roo
 	len = strlen(newpass);
 	for (i = 0; i < len; i++) {
 		if (iscntrl((int)newpass[i])) {
-			DEBUG(0,
-			      ("chat_with_program: newpass contains control characters (disallowed).\n"));
+			DEBUG(0, ("chgpasswd: newpass contains control characters (disallowed).\n"));
 			return False;
 		}
 	}
-
+	
 #ifdef WITH_PAM
 	if (lp_pam_password_change()) {
 		BOOL ret;
@@ -513,8 +497,12 @@ BOOL chgpasswd(char *name, const char *oldpass, const char *newpass, BOOL as_roo
 		if (as_root)
 			become_root();
 
-		ret = smb_pam_passchange(name, oldpass, newpass);
-
+		if (pass) {
+			ret = smb_pam_passchange(pass->pw_name, oldpass, newpass);
+		} else {
+			ret = smb_pam_passchange(name, oldpass, newpass);
+		}
+			
 		if (as_root)
 			unbecome_root();
 
@@ -522,6 +510,13 @@ BOOL chgpasswd(char *name, const char *oldpass, const char *newpass, BOOL as_roo
 	}
 #endif
 
+	/* A non-PAM password change just doen't make sense without a valid local user */
+
+	if (pass == NULL) {
+		DEBUG(0, ("chgpasswd: user %s doesn't exist in the UNIX password database.\n", name));
+		return False;
+	}
+
 	pstrcpy(passwordprogram, lp_passwd_program());
 	pstrcpy(chatsequence, lp_passwd_chat());
 
@@ -537,7 +532,7 @@ BOOL chgpasswd(char *name, const char *oldpass, const char *newpass, BOOL as_roo
 
 	if (as_root) {
 		/* The password program *must* contain the user name to work. Fail if not. */
-		if (strstr(passwordprogram, "%u") == NULL) {
+		if (strstr_m(passwordprogram, "%u") == NULL) {
 			DEBUG(0,("chgpasswd: Running as root the 'passwd program' parameter *MUST* contain \
 the string %%u, and the given string %s does not.\n", passwordprogram ));
 			return False;
@@ -553,14 +548,15 @@ the string %%u, and the given string %s does not.\n", passwordprogram ));
 	all_string_sub(chatsequence, "%o", oldpass, sizeof(pstring));
 	all_string_sub(chatsequence, "%n", newpass, sizeof(pstring));
 	return (chat_with_program
-		(passwordprogram, name, chatsequence, as_root));
+		(passwordprogram, pass, chatsequence, as_root));
 }
 
 #else /* ALLOW_CHANGE_PASSWORD */
 
-BOOL chgpasswd(char *name, const char *oldpass, const char *newpass, BOOL as_root)
+BOOL chgpasswd(const char *name, const struct passwd *pass, 
+	       const char *oldpass, const char *newpass, BOOL as_root)
 {
-	DEBUG(0, ("Password changing not compiled in (user=%s)\n", name));
+	DEBUG(0, ("chgpasswd: Unix Password changing not compiled in (user=%s)\n", name));
 	return (False);
 }
 #endif /* ALLOW_CHANGE_PASSWORD */
@@ -572,12 +568,11 @@ BOOL chgpasswd(char *name, const char *oldpass, const char *newpass, BOOL as_roo
 BOOL check_lanman_password(char *user, uchar * pass1,
 			   uchar * pass2, SAM_ACCOUNT **hnd)
 {
-	static uchar null_pw[16];
 	uchar unenc_new_pw[16];
 	uchar unenc_old_pw[16];
 	SAM_ACCOUNT *sampass = NULL;
 	uint16 acct_ctrl;
-	uint8 *lanman_pw;
+	const uint8 *lanman_pw;
 	BOOL ret;
 	
 	become_root();
@@ -586,7 +581,7 @@ BOOL check_lanman_password(char *user, uchar * pass1,
 
 	if (ret == False) {
 		DEBUG(0,("check_lanman_password: getsampwnam returned NULL\n"));
-		pdb_free_sam(sampass);
+		pdb_free_sam(&sampass);
 		return False;
 	}
 	
@@ -595,24 +590,21 @@ BOOL check_lanman_password(char *user, uchar * pass1,
 
 	if (acct_ctrl & ACB_DISABLED) {
 		DEBUG(0,("check_lanman_password: account %s disabled.\n", user));
-		pdb_free_sam(sampass);
+		pdb_free_sam(&sampass);
 		return False;
 	}
 
-	if ((lanman_pw == NULL) && (acct_ctrl & ACB_PWNOTREQ)) {
-		uchar no_pw[14];
-		memset(no_pw, '\0', 14);
-		E_P16(no_pw, null_pw);
-		if (!pdb_set_lanman_passwd(sampass, null_pw)) {
-			pdb_free_sam(sampass);
+	if (lanman_pw == NULL) {
+		if (acct_ctrl & ACB_PWNOTREQ) {
+			/* this saves the pointer for the caller */
+			*hnd = sampass;
+			return True;
+		} else {
+			DEBUG(0, ("check_lanman_password: no lanman password !\n"));
+			pdb_free_sam(&sampass);
 			return False;
 		}
 	}
-	else if (lanman_pw == NULL) {
-		DEBUG(0, ("check_lanman_password: no lanman password !\n"));
-		pdb_free_sam(sampass);
-		return False;
-	}
 
 	/* Get the new lanman hash. */
 	D_P16(lanman_pw, pass2, unenc_new_pw);
@@ -623,13 +615,12 @@ BOOL check_lanman_password(char *user, uchar * pass1,
 	/* Check that the two old passwords match. */
 	if (memcmp(lanman_pw, unenc_old_pw, 16)) {
 		DEBUG(0,("check_lanman_password: old password doesn't match.\n"));
-		pdb_free_sam(sampass);
+		pdb_free_sam(&sampass);
 		return False;
 	}
 
 	/* this saves the pointer for the caller */
 	*hnd = sampass;
-	
 	return True;
 }
 
@@ -637,16 +628,17 @@ BOOL check_lanman_password(char *user, uchar * pass1,
  Code to change the lanman hashed password.
  It nulls out the NT hashed password as it will
  no longer be valid.
+ NOTE this function is designed to be called as root. Check the old password
+ is correct before calling. JRA.
 ************************************************************/
 
-BOOL change_lanman_password(SAM_ACCOUNT *sampass, uchar * pass1,
-			    uchar * pass2)
+BOOL change_lanman_password(SAM_ACCOUNT *sampass, uchar *pass2)
 {
 	static uchar null_pw[16];
 	uchar unenc_new_pw[16];
 	BOOL ret;
 	uint16 acct_ctrl;
-	uint8 *pwd;
+	const uint8 *pwd;
 
 	if (sampass == NULL) {
 		DEBUG(0,("change_lanman_password: no smb password entry.\n"));
@@ -662,29 +654,39 @@ BOOL change_lanman_password(SAM_ACCOUNT *sampass, uchar * pass1,
 		return False;
 	}
 
-	if ((pwd == NULL) && (acct_ctrl & ACB_PWNOTREQ)) {
-		uchar no_pw[14];
-		memset(no_pw, '\0', 14);
-		E_P16(no_pw, null_pw);
-		if (!pdb_set_lanman_passwd(sampass, null_pw))
+	if (pwd == NULL) { 
+		if (acct_ctrl & ACB_PWNOTREQ) {
+			uchar no_pw[14];
+			memset(no_pw, '\0', 14);
+			E_P16(no_pw, null_pw);
+
+			/* Get the new lanman hash. */
+			D_P16(null_pw, pass2, unenc_new_pw);
+		} else {
+			DEBUG(0,("change_lanman_password: no lanman password !\n"));
 			return False;
+		}
+	} else {
+		/* Get the new lanman hash. */
+		D_P16(pwd, pass2, unenc_new_pw);
 	}
-	else if (pwd == NULL) {
-		DEBUG(0,("change_lanman_password: no lanman password !\n"));
+
+	if (!pdb_set_lanman_passwd(sampass, unenc_new_pw, PDB_CHANGED)) {
 		return False;
 	}
 
-	/* Get the new lanman hash. */
-	D_P16(pwd, pass2, unenc_new_pw);
-
-	if (!pdb_set_lanman_passwd(sampass, unenc_new_pw))
-		return False;
-	pdb_set_nt_passwd    (sampass, NULL);	/* We lose the NT hash. Sorry. */
+	if (!pdb_set_nt_passwd    (sampass, NULL, PDB_CHANGED)) {
+		return False;	/* We lose the NT hash. Sorry. */
+	}
 
+	if (!pdb_set_pass_changed_now  (sampass)) {
+		pdb_free_sam(&sampass);
+		/* Not quite sure what this one qualifies as, but this will do */
+		return False; 
+	}
+ 
 	/* Now flush the sam_passwd struct to persistent storage */
-	become_root();
-	ret = pdb_update_sam_account (sampass, False);
-	unbecome_root();
+	ret = pdb_update_sam_account (sampass);
 
 	return ret;
 }
@@ -692,65 +694,73 @@ BOOL change_lanman_password(SAM_ACCOUNT *sampass, uchar * pass1,
 /***********************************************************
  Code to check and change the OEM hashed password.
 ************************************************************/
-BOOL pass_oem_change(char *user,
-		     uchar * lmdata, uchar * lmhash,
-		     uchar * ntdata, uchar * nthash)
+
+NTSTATUS pass_oem_change(char *user,
+			 uchar password_encrypted_with_lm_hash[516], 
+			 const uchar old_lm_hash_encrypted[16],
+			 uchar password_encrypted_with_nt_hash[516], 
+			 const uchar old_nt_hash_encrypted[16])
 {
-	fstring new_passwd;
+	pstring new_passwd;
 	SAM_ACCOUNT *sampass = NULL;
-	BOOL ret = check_oem_password(user, lmdata, lmhash, ntdata, nthash,
-				      &sampass, new_passwd, sizeof(new_passwd));
+	NTSTATUS nt_status = check_oem_password(user, password_encrypted_with_lm_hash, 
+						old_lm_hash_encrypted, 
+						password_encrypted_with_nt_hash, 
+						old_nt_hash_encrypted,
+						&sampass, new_passwd, sizeof(new_passwd));
+	
+	if (!NT_STATUS_IS_OK(nt_status))
+		return nt_status;
 
-	/* 
-	 * At this point we have the new case-sensitive plaintext
-	 * password in the fstring new_passwd. If we wanted to synchronise
-	 * with UNIX passwords we would call a UNIX password changing 
-	 * function here. However it would have to be done as root
-	 * as the plaintext of the old users password is not 
-	 * available. JRA.
-	 */
-
-	if (ret && lp_unix_password_sync())
-		ret = chgpasswd(user, "", new_passwd, True);
-
-	if (ret)
-		ret = change_oem_password(sampass, new_passwd, False);
+	/* We've already checked the old password here.... */
+	become_root();
+	nt_status = change_oem_password(sampass, NULL, new_passwd, True);
+	unbecome_root();
 
 	memset(new_passwd, 0, sizeof(new_passwd));
 
-	pdb_free_sam(sampass);
+	pdb_free_sam(&sampass);
 
-	return ret;
+	return nt_status;
 }
 
 /***********************************************************
- Code to check the OEM hashed password.
+ Decrypt and verify a user password change.  
 
- this function ignores the 516 byte nt OEM hashed password
- but does use the lm OEM password to check the nt hashed-hash.
+ The 516 byte long buffers are encrypted with the old NT and 
+ old LM passwords, and if the NT passwords are present, both 
+ buffers contain a unicode string.
 
+ After decrypting the buffers, check the password is correct by
+ matching the old hashed passwords with the passwords in the passdb.
+ 
 ************************************************************/
 
-BOOL check_oem_password(char *user,
-			uchar * lmdata, uchar * lmhash,
-			uchar * ntdata, uchar * nthash,
-			SAM_ACCOUNT **hnd, char *new_passwd,
-			int new_passwd_size)
+static NTSTATUS check_oem_password(const char *user,
+				   uchar password_encrypted_with_lm_hash[516], 
+				   const uchar old_lm_hash_encrypted[16],
+				   uchar password_encrypted_with_nt_hash[516], 
+				   const uchar old_nt_hash_encrypted[16],
+				   SAM_ACCOUNT **hnd, char *new_passwd,
+				   int new_passwd_size)
 {
 	static uchar null_pw[16];
 	static uchar null_ntpw[16];
 	SAM_ACCOUNT *sampass = NULL;
-	uint8 *lanman_pw, *nt_pw;
+	char *password_encrypted;
+	const char *encryption_key;
+	const uint8 *lanman_pw, *nt_pw;
 	uint16 acct_ctrl;
-	int new_pw_len;
-	uchar new_ntp16[16];
-	uchar unenc_old_ntpw[16];
-	uchar new_p16[16];
-	uchar unenc_old_pw[16];
+	uint32 new_pw_len;
+	uchar new_nt_hash[16];
+	uchar old_nt_hash_plain[16];
+	uchar new_lm_hash[16];
+	uchar old_lm_hash_plain[16];
 	char no_pw[2];
 	BOOL ret;
 
-	BOOL nt_pass_set = (ntdata != NULL && nthash != NULL);
+	BOOL nt_pass_set = (password_encrypted_with_nt_hash && old_nt_hash_encrypted);
+	BOOL lm_pass_set = (password_encrypted_with_lm_hash && old_lm_hash_encrypted);
 
 	*hnd = NULL;
 
@@ -762,90 +772,72 @@ BOOL check_oem_password(char *user,
 
 	if (ret == False) {
 		DEBUG(0, ("check_oem_password: getsmbpwnam returned NULL\n"));
-		pdb_free_sam(sampass);
-		return False;
+		pdb_free_sam(&sampass);
+		return NT_STATUS_NO_SUCH_USER; 
 	}
 
 	acct_ctrl = pdb_get_acct_ctrl(sampass);
 	
 	if (acct_ctrl & ACB_DISABLED) {
-		DEBUG(0,("check_lanman_password: account %s disabled.\n", user));
-		pdb_free_sam(sampass);
-		return False;
+		DEBUG(2,("check_lanman_password: account %s disabled.\n", user));
+		pdb_free_sam(&sampass);
+		return NT_STATUS_ACCOUNT_DISABLED;
 	}
 
-	/* construct a null password (in case one is needed */
-	no_pw[0] = 0;
-	no_pw[1] = 0;
-	nt_lm_owf_gen(no_pw, null_ntpw, null_pw);
+	if (acct_ctrl & ACB_PWNOTREQ && lp_null_passwords()) {
+		/* construct a null password (in case one is needed */
+		no_pw[0] = 0;
+		no_pw[1] = 0;
+		nt_lm_owf_gen(no_pw, null_ntpw, null_pw);
+		lanman_pw = null_pw;
+		nt_pw = null_pw;
 
-	/* save pointers to passwords so we don't have to keep looking them up */
-	lanman_pw = pdb_get_lanman_passwd(sampass);
-	nt_pw = pdb_get_nt_passwd(sampass);
-
-	/* check for null passwords */
-	if (lanman_pw == NULL) {
-		if (acct_ctrl & ACB_PWNOTREQ) {
-			if (!pdb_set_lanman_passwd(sampass, null_pw)) {
-				pdb_free_sam(sampass);
-				return False;
-			}
-			lanman_pw = pdb_get_lanman_passwd(sampass);
-			if (!lanman_pw) {
-				pdb_free_sam(sampass);
-				return False;
-			}
-		} else {
-			DEBUG(0,("check_oem_password: no lanman password !\n"));
-			pdb_free_sam(sampass);
-			return False;
-		}
-	}
-
-	if (pdb_get_nt_passwd(sampass) == NULL && nt_pass_set) {
-		if (acct_ctrl & ACB_PWNOTREQ) {
-			pdb_set_nt_passwd(sampass, null_ntpw);
-			nt_pw = pdb_get_nt_passwd(sampass);
-			if (!nt_pw) {
-				pdb_free_sam(sampass);
-				return False;
-			}
-		} else {
-			DEBUG(0,("check_oem_password: no ntlm password !\n"));
-			pdb_free_sam(sampass);
-			return False;
-		}
-	}
-
-	/* 
-	 * Call the hash function to get the new password.
-	 */
-	SamOEMhash((uchar *) lmdata, (uchar *)lanman_pw, 516);
-
-	/* 
-	 * The length of the new password is in the last 4 bytes of
-	 * the data buffer.
-	 */
-
-	new_pw_len = IVAL(lmdata, 512);
-	if (new_pw_len < 0 || new_pw_len > new_passwd_size - 1) {
-		DEBUG(0,("check_oem_password: incorrect password length (%d).\n", new_pw_len));
-		pdb_free_sam(sampass);
-		return False;
-	}
-
-	if (nt_pass_set) {
-		/*
-		 * nt passwords are in unicode
-		 */
-		int uni_pw_len = new_pw_len;
-		char *pw;
-		new_pw_len /= 2;
-		pw = dos_unistrn2((uint16 *)(&lmdata[512 - uni_pw_len]), new_pw_len);
-		memcpy(new_passwd, pw, new_pw_len + 1);
 	} else {
-		memcpy(new_passwd, &lmdata[512 - new_pw_len], new_pw_len);
-		new_passwd[new_pw_len] = 0;
+		/* save pointers to passwords so we don't have to keep looking them up */
+		if (lp_lanman_auth()) {
+			lanman_pw = pdb_get_lanman_passwd(sampass);
+		} else {
+			lanman_pw = NULL;
+		}
+		nt_pw = pdb_get_nt_passwd(sampass);
+	}
+
+	if (nt_pw && nt_pass_set) {
+		/* IDEAL Case: passwords are in unicode, and we can
+		 * read use the password encrypted with the NT hash 
+		 */
+		password_encrypted = password_encrypted_with_nt_hash;
+		encryption_key = nt_pw;
+	} else if (lanman_pw && lm_pass_set) {
+		/* password may still be in unicode, but use LM hash version */
+		password_encrypted = password_encrypted_with_lm_hash;
+		encryption_key = lanman_pw;
+	} else if (nt_pass_set) {
+		DEBUG(1, ("NT password change supplied for user %s, but we have no NT password to check it with\n", 
+			  user));
+		pdb_free_sam(&sampass);
+		return NT_STATUS_WRONG_PASSWORD;	
+	} else if (lm_pass_set) {
+		DEBUG(1, ("LM password change supplied for user %s, but we have no LanMan password to check it with\n", 
+			  user));
+		pdb_free_sam(&sampass);
+		return NT_STATUS_WRONG_PASSWORD;
+	} else {
+		DEBUG(1, ("password change requested for user %s, but no password supplied!\n", 
+			  user));
+		pdb_free_sam(&sampass);
+		return NT_STATUS_WRONG_PASSWORD;
+	}
+
+	/* 
+	 * Decrypt the password with the key 
+	 */
+	SamOEMhash( password_encrypted, encryption_key, 516);
+
+	if ( !decode_pw_buffer(password_encrypted, new_passwd, new_passwd_size, &new_pw_len, 
+			       nt_pass_set ? STR_UNICODE : STR_ASCII)) {
+		pdb_free_sam(&sampass);
+		return NT_STATUS_WRONG_PASSWORD;
 	}
 
 	/*
@@ -853,135 +845,163 @@ BOOL check_oem_password(char *user,
 	 * use it as a key to test the passed old password.
 	 */
 
-	nt_lm_owf_gen(new_passwd, new_ntp16, new_p16);
+	if (nt_pass_set) {
+		/* NT passwords, verify the NT hash. */
+		
+		/* Calculate the MD4 hash (NT compatible) of the password */
+		memset(new_nt_hash, '\0', 16);
+		E_md4hash(new_passwd, new_nt_hash);
+
+		if (nt_pw) {
+			/*
+			 * Now use new_nt_hash as the key to see if the old
+			 * password matches.
+			 */
+			D_P16(new_nt_hash, old_nt_hash_encrypted, old_nt_hash_plain);
+			
+			if (memcmp(nt_pw, old_nt_hash_plain, 16)) {
+				DEBUG(0,("check_oem_password: old lm password doesn't match.\n"));
+				pdb_free_sam(&sampass);
+				return NT_STATUS_WRONG_PASSWORD;
+			}
+			
+			/* We could check the LM password here, but there is
+			 * little point, we already know the password is
+			 * correct, and the LM password might not even be
+			 * present. */
+
+			/* Further, LM hash generation algorithms
+			 * differ with charset, so we could
+			 * incorrectly fail a perfectly valid password
+			 * change */
+#ifdef DEBUG_PASSWORD
+			DEBUG(100,
+			      ("check_oem_password: password %s ok\n", new_passwd));
+#endif
+			*hnd = sampass;
+			return NT_STATUS_OK;
+		}
+		
+		if (lanman_pw) {
+			/*
+			 * Now use new_nt_hash as the key to see if the old
+			 * LM password matches.
+			 */
+			D_P16(new_nt_hash, old_lm_hash_encrypted, old_lm_hash_plain);
+			
+			if (memcmp(lanman_pw, old_lm_hash_plain, 16)) {
+				DEBUG(0,("check_oem_password: old lm password doesn't match.\n"));
+				pdb_free_sam(&sampass);
+				return NT_STATUS_WRONG_PASSWORD;
+			}
+#ifdef DEBUG_PASSWORD
+			DEBUG(100,
+			      ("check_oem_password: password %s ok\n", new_passwd));
+#endif
+			*hnd = sampass;
+			return NT_STATUS_OK;
+		}
+	}
+
+	if (lanman_pw && lm_pass_set) {
+
+		E_deshash(new_passwd, new_lm_hash);
 
-	if (!nt_pass_set) {
 		/*
-		 * Now use new_p16 as the key to see if the old
+		 * Now use new_lm_hash as the key to see if the old
 		 * password matches.
 		 */
-		D_P16(new_p16, lmhash, unenc_old_pw);
-
-		if (memcmp(lanman_pw, unenc_old_pw, 16)) {
+		D_P16(new_lm_hash, old_lm_hash_encrypted, old_lm_hash_plain);
+		
+		if (memcmp(lanman_pw, old_lm_hash_plain, 16)) {
 			DEBUG(0,("check_oem_password: old lm password doesn't match.\n"));
-			pdb_free_sam(sampass);
-			return False;
+			pdb_free_sam(&sampass);
+			return NT_STATUS_WRONG_PASSWORD;
 		}
-
+		
 #ifdef DEBUG_PASSWORD
 		DEBUG(100,
 		      ("check_oem_password: password %s ok\n", new_passwd));
 #endif
 		*hnd = sampass;
-		return True;
+		return NT_STATUS_OK;
 	}
 
-	/*
-	 * Now use new_p16 as the key to see if the old
-	 * password matches.
-	 */
-	D_P16(new_ntp16, lmhash, unenc_old_pw);
-	D_P16(new_ntp16, nthash, unenc_old_ntpw);
-
-	if (memcmp(lanman_pw, unenc_old_pw, 16)) {
-		DEBUG(0,("check_oem_password: old lm password doesn't match.\n"));
-		pdb_free_sam(sampass);
-		return False;
-	}
-
-	if (memcmp(nt_pw, unenc_old_ntpw, 16)) {
-		DEBUG(0,("check_oem_password: old nt password doesn't match.\n"));
-		pdb_free_sam(sampass);
-		return False;
-	}
-#ifdef DEBUG_PASSWORD
-	DEBUG(100, ("check_oem_password: password %s ok\n", new_passwd));
-#endif
-	*hnd = sampass;
-	return True;
+	/* should not be reached */
+	pdb_free_sam(&sampass);
+	return NT_STATUS_WRONG_PASSWORD;
 }
 
 /***********************************************************
  Code to change the oem password. Changes both the lanman
- and NT hashes.
- override = False, normal
- override = True, override XXXXXXXXXX'd password
+ and NT hashes.  Old_passwd is almost always NULL.
+ NOTE this function is designed to be called as root. Check the old password
+ is correct before calling. JRA.
 ************************************************************/
 
-BOOL change_oem_password(SAM_ACCOUNT *hnd, char *new_passwd,
-			 BOOL override)
+NTSTATUS change_oem_password(SAM_ACCOUNT *hnd, char *old_passwd, char *new_passwd, BOOL as_root)
 {
+	struct passwd *pass;
+
 	BOOL ret;
-	uchar new_nt_p16[16];
-	uchar new_p16[16];
+	uint32 min_len;
 
-	nt_lm_owf_gen(new_passwd, new_nt_p16, new_p16);
+	if (time(NULL) < pdb_get_pass_can_change_time(hnd)) {
+		DEBUG(1, ("user %s cannot change password now, must wait until %s\n", 
+			  pdb_get_username(hnd), http_timestring(pdb_get_pass_can_change_time(hnd))));
+		return NT_STATUS_PASSWORD_RESTRICTION;
+	}
 
-	if (!pdb_set_lanman_passwd (hnd, new_p16))
-		return False;
-	if (!pdb_set_nt_passwd(hnd, new_nt_p16))
-		return False;
+	if (account_policy_get(AP_MIN_PASSWORD_LEN, &min_len) && (strlen(new_passwd) < min_len)) {
+		DEBUG(1, ("user %s cannot change password - password too short\n", 
+			  pdb_get_username(hnd)));
+		DEBUGADD(1, (" account policy min password len = %d\n", min_len));
+		return NT_STATUS_PASSWORD_RESTRICTION;
+/* 		return NT_STATUS_PWD_TOO_SHORT; */
+	}
+
+	/* Take the passed information and test it for minimum criteria */
+	/* Minimum password length */
+	if (strlen(new_passwd) < lp_min_passwd_length()) {
+		/* too short, must be at least MINPASSWDLENGTH */
+		DEBUG(1, ("Password Change: user %s, New password is shorter than minimum password length = %d\n",
+		       pdb_get_username(hnd), lp_min_passwd_length()));
+		return NT_STATUS_PASSWORD_RESTRICTION;
+/* 		return NT_STATUS_PWD_TOO_SHORT; */
+	}
+
+	pass = Get_Pwnam(pdb_get_username(hnd));
+	if (!pass) {
+		DEBUG(1, ("check_oem_password: Username does not exist in system !?!\n"));
+	}
+
+	/*
+	 * If unix password sync was requested, attempt to change
+	 * the /etc/passwd database first. Return failure if this cannot
+	 * be done.
+	 *
+	 * This occurs before the oem change, because we don't want to
+	 * update it if chgpasswd failed.
+	 *
+	 * Conditional on lp_unix_password_sync() because we don't want
+	 * to touch the unix db unless we have admin permission.
+	 */
+	
+	if(lp_unix_password_sync() &&
+		!chgpasswd(pdb_get_username(hnd), pass, old_passwd, new_passwd, as_root)) {
+		return NT_STATUS_ACCESS_DENIED;
+	}
+
+	if (!pdb_set_plaintext_passwd (hnd, new_passwd)) {
+		return NT_STATUS_ACCESS_DENIED;
+	}
 
 	/* Now write it into the file. */
-	become_root();
-	ret = pdb_update_sam_account (hnd, override);
-	unbecome_root();
+	ret = pdb_update_sam_account (hnd);
 
-	memset(new_passwd, '\0', strlen(new_passwd));
-
-	return ret;
-}
-
-/***********************************************************
- Code to check a plaintext password against smbpasswd entries.
-***********************************************************/
-
-BOOL check_plaintext_password(char *user, char *old_passwd,
-			      int old_passwd_size, SAM_ACCOUNT **hnd)
-{
-	SAM_ACCOUNT  *sampass = NULL;
-	uchar old_pw[16], old_ntpw[16];
-	BOOL ret;
-
-	*hnd = NULL;
-
-	pdb_init_sam(&sampass);
-
-	become_root();
-	ret = pdb_getsampwnam(sampass, user);
-	unbecome_root();
-
-	if (ret == False) {
-		DEBUG(0,("check_plaintext_password: getsmbpwnam returned NULL\n"));
-		pdb_free_sam(sampass);
-		return False;
+	if (!ret) {
+		return NT_STATUS_ACCESS_DENIED;
 	}
-
-	if (pdb_get_acct_ctrl(sampass) & ACB_DISABLED) {
-		DEBUG(0,("check_plaintext_password: account %s disabled.\n", user));
-		pdb_free_sam(sampass);
-		return (False);
-	}
-
-	nt_lm_owf_gen(old_passwd, old_ntpw, old_pw);
-
-#ifdef DEBUG_PASSWORD
-	DEBUG(100, ("check_plaintext_password: nt_passwd \n"));
-	dump_data(100, pdb_get_nt_passwd(sampass), 16);
-	DEBUG(100, ("check_plaintext_password: old_ntpw \n"));
-	dump_data(100, old_ntpw, 16);
-	DEBUG(100, ("check_plaintext_password: lanman_passwd \n"));
-	dump_data(100, pdb_get_lanman_passwd(sampass), 16);
-	DEBUG(100, ("check_plaintext_password: old_pw\n"));
-	dump_data(100, old_pw, 16);
-#endif
-
-	if (memcmp(pdb_get_nt_passwd(sampass), old_ntpw, 16)
-	    && memcmp(pdb_get_lanman_passwd(sampass), old_pw, 16)) {
-		pdb_free_sam(sampass);
-		return (False);
-	} else {
-		*hnd = sampass;
-		return (True);
-	}
+	
+	return NT_STATUS_OK;
 }
