@@ -23,12 +23,13 @@
 static struct mangle_fns *mangle_fns;
 
 /* this allows us to add more mangling backends */
-static struct {
+static const struct {
 	const char *name;
 	struct mangle_fns *(*init_fn)(void);
 } mangle_backends[] = {
 	{ "hash", mangle_hash_init },
 	{ "hash2", mangle_hash2_init },
+	/*{ "tdb", mangle_tdb_init }, */
 	{ NULL, NULL }
 };
 
@@ -97,16 +98,16 @@ BOOL mangle_is_8_3_wildcards(const char *fname, BOOL check_case)
   looking for a matching name if it doesn't. It should succeed most of the time
   or there will be a huge performance penalty
 */
-BOOL mangle_check_cache(char *s, size_t maxlen)
+BOOL mangle_check_cache(char *s)
 {
-	return mangle_fns->check_cache(s, maxlen);
+	return mangle_fns->check_cache(s);
 }
 
 /* 
    map a long filename to a 8.3 name. 
  */
 
-void mangle_map(char *OutName, BOOL need83, BOOL cache83, int snum)
+void mangle_map(pstring OutName, BOOL need83, BOOL cache83, int snum)
 {
 	/* name mangling can be disabled for speed, in which case
 	   we just truncate the string */
