@@ -53,6 +53,12 @@
 
 #include "includes.h"
 
+#if 1
+#define M_DEBUG(level, x) DEBUG(level, x)
+#else
+#define M_DEBUG(level, x)
+#endif
+
 /* these flags are used to mark characters in as having particular
    properties */
 #define FLAG_BASECHAR 1
@@ -81,6 +87,13 @@ static unsigned char char_flags[256];
 
 #define FLAG_CHECK(c, flag) (char_flags[(unsigned char)(c)] & (flag))
 
+/*
+  this determines how many characters are used from the original filename
+  in the 8.3 mangled name. A larger value leads to a weaker hash and more collisions.
+  The largest possible value is 6.
+*/
+static unsigned mangle_prefix;
+
 /* we will use a very simple direct mapped prefix cache. The big
    advantage of this cache structure is speed and low memory usage 
 
@@ -118,7 +131,7 @@ static u32 mangle_hash(const char *key, unsigned length)
 	   function */
 	strncpy(str, key, length);
 	str[length] = 0;
-	strupper(str);
+	strupper_m(str);
 
 	/* the length of a multi-byte string can change after a strupper_m */
 	length = strlen(str);
@@ -188,45 +201,49 @@ static const char *cache_lookup(u32 hash)
    In this algorithm, mangled names use only pure ascii characters (no
    multi-byte) so we can avoid doing a UCS2 conversion 
  */
-static BOOL is_mangled_component(const char *name)
+static BOOL is_mangled_component(const char *name, size_t len)
 {
-	int len, i;
+	unsigned int i;
 
-	DEBUG(10,("is_mangled_component %s ?\n", name));
+	M_DEBUG(10,("is_mangled_component %s (len %u) ?\n", name, (unsigned int)len));
 
 	/* check the length */
-	len = strlen(name);
-	if (len > 12 || len < 8) return False;
+	if (len > 12 || len < 8)
+		return False;
 
 	/* the best distinguishing characteristic is the ~ */
-	if (len > 7 && name[6] != '~') return False;
+	if (name[6] != '~')
+		return False;
 
 	/* check extension */
 	if (len > 8) {
-		if (name[8] != '.') return False;
-		for (i=9; name[i]; i++) {
+		if (name[8] != '.')
+			return False;
+		for (i=9; name[i] && i < len; i++) {
 			if (! FLAG_CHECK(name[i], FLAG_ASCII)) {
 				return False;
 			}
 		}
 	}
 	
-	/* check first character */
-	if (! FLAG_CHECK(name[0], FLAG_ASCII)) {
-		return False;
+	/* check lead characters */
+	for (i=0;i<mangle_prefix;i++) {
+		if (! FLAG_CHECK(name[i], FLAG_ASCII)) {
+			return False;
+		}
 	}
 	
 	/* check rest of hash */
 	if (! FLAG_CHECK(name[7], FLAG_BASECHAR)) {
 		return False;
 	}
-	for (i=1;i<6;i++) {
+	for (i=mangle_prefix;i<6;i++) {
 		if (! FLAG_CHECK(name[i], FLAG_BASECHAR)) {
 			return False;
 		}
 	}
 
-	DEBUG(10,("is_mangled %s -> yes\n", name));
+	M_DEBUG(10,("is_mangled_component %s (len %u) -> yes\n", name, (unsigned int)len));
 
 	return True;
 }
@@ -249,19 +266,16 @@ static BOOL is_mangled(const char *name)
 	const char *p;
 	const char *s;
 
-	DEBUG(10,("is_mangled %s ?\n", name));
+	M_DEBUG(10,("is_mangled %s ?\n", name));
 
 	for (s=name; (p=strchr(s, '/')); s=p+1) {
-		char *component = strndup(s, PTR_DIFF(p, s));
-		if (is_mangled_component(component)) {
-			free(component);
+		if (is_mangled_component(s, PTR_DIFF(p, s))) {
 			return True;
 		}
-		free(component);
 	}
 	
 	/* and the last part ... */
-	return is_mangled_component(s);
+	return is_mangled_component(s,strlen(s));
 }
 
 
@@ -290,7 +304,8 @@ static BOOL is_8_3(const char *name, BOOL check_case, BOOL allow_wildcards)
 	 the result we need in this case. Using strlen_m would not
 	 only be slower, it would be incorrect */
 	len = strlen(name);
-	if (len > 12) return False;
+	if (len > 12)
+		return False;
 
 	/* find the '.'. Note that once again we use the non-multibyte
            function */
@@ -310,7 +325,7 @@ static BOOL is_8_3(const char *name, BOOL check_case, BOOL allow_wildcards)
 		prefix_len = PTR_DIFF(dot_p, name);
 		suffix_len = len - (prefix_len+1);
 
-		if (prefix_len > 8 || suffix_len > 3) {
+		if (prefix_len > 8 || suffix_len > 3 || suffix_len == 0) {
 			return False;
 		}
 
@@ -350,22 +365,22 @@ static void mangle_reset(void)
 
   The filename must be able to hold at least sizeof(fstring) 
 */
-static BOOL check_cache(char *name, size_t maxlen)
+static BOOL check_cache(char *name)
 {
 	u32 hash, multiplier;
-	int i;
+	unsigned int i;
 	const char *prefix;
 	char extension[4];
 
 	/* make sure that this is a mangled name from this cache */
 	if (!is_mangled(name)) {
-		DEBUG(10,("check_cache: %s -> not mangled\n", name));
+		M_DEBUG(10,("check_cache: %s -> not mangled\n", name));
 		return False;
 	}
 
 	/* we need to extract the hash from the 8.3 name */
 	hash = base_reverse[(unsigned char)name[7]];
-	for (multiplier=36, i=5;i>=1;i--) {
+	for (multiplier=36, i=5;i>=mangle_prefix;i--) {
 		u32 v = base_reverse[(unsigned char)name[i]];
 		hash += multiplier * v;
 		multiplier *= 36;
@@ -374,7 +389,7 @@ static BOOL check_cache(char *name, size_t maxlen)
 	/* now look in the prefix cache for that hash */
 	prefix = cache_lookup(hash);
 	if (!prefix) {
-		DEBUG(10,("check_cache: %s -> %08X -> not found\n", name, hash));
+		M_DEBUG(10,("check_cache: %s -> %08X -> not found\n", name, hash));
 		return False;
 	}
 
@@ -387,11 +402,11 @@ static BOOL check_cache(char *name, size_t maxlen)
 	}
 
 	if (extension[0]) {
-		DEBUG(10,("check_cache: %s -> %s.%s\n", name, prefix, extension));
-		slprintf(name, maxlen, "%s.%s", prefix, extension);
+		M_DEBUG(10,("check_cache: %s -> %s.%s\n", name, prefix, extension));
+		slprintf(name, sizeof(fstring), "%s.%s", prefix, extension);
 	} else {
-		DEBUG(10,("check_cache: %s -> %s\n", name, prefix));
-		safe_strcpy(name, prefix, maxlen);
+		M_DEBUG(10,("check_cache: %s -> %s\n", name, prefix));
+		fstrcpy(name, prefix);
 	}
 
 	return True;
@@ -412,7 +427,7 @@ static BOOL is_reserved_name(const char *name)
 		for (i=0; reserved_names[i]; i++) {
 			int len = strlen(reserved_names[i]);
 			/* note that we match on COM1 as well as COM1.foo */
-			if (strncasecmp(name, reserved_names[i], len) == 0 &&
+			if (strnequal(name, reserved_names[i], len) &&
 			    (name[len] == '.' || name[len] == 0)) {
 				return True;
 			}
@@ -434,6 +449,23 @@ static BOOL is_legal_name(const char *name)
 	size_t numdots = 0;
 
 	while (*name) {
+		if (((unsigned int)name[0]) > 128 && (name[1] != 0)) {
+			/* Possible start of mb character. */
+			char mbc[2];
+			/*
+			 * Note that if CH_UNIX is utf8 a string may be 3
+			 * bytes, but this is ok as mb utf8 characters don't
+			 * contain embedded ascii bytes. We are really checking
+			 * for mb UNIX asian characters like Japanese (SJIS) here.
+			 * JRA.
+			 */
+			if (convert_string(CH_UNIX, CH_UCS2, name, 2, mbc, 2, False) == 2) {
+				/* Was a good mb string. */
+				name += 2;
+				continue;
+			}
+		}
+
 		if (FLAG_CHECK(name[0], FLAG_ILLEGAL)) {
 			return False;
 		}
@@ -469,13 +501,13 @@ static BOOL is_legal_name(const char *name)
 
   the name parameter must be able to hold 13 bytes
 */
-static void name_map(char *name, BOOL need83, BOOL cache83)
+static void name_map(fstring name, BOOL need83, BOOL cache83)
 {
 	char *dot_p;
-	char lead_char;
+	char lead_chars[7];
 	char extension[4];
-	int extension_length, i;
-	int prefix_len;
+	unsigned int extension_length, i;
+	unsigned int prefix_len;
 	u32 hash, v;
 	char new_name[13];
 
@@ -510,15 +542,20 @@ static void name_map(char *name, BOOL need83, BOOL cache83)
 		if (i == 0 || i == 4) dot_p = NULL;
 	}
 
-	/* the leading character in the mangled name is taken from
-	   the first character of the name, if it is ascii 
-	   otherwise '_' is used
+	/* the leading characters in the mangled name is taken from
+	   the first characters of the name, if they are ascii otherwise
+	   '_' is used
 	*/
-	lead_char = name[0];
-	if (! FLAG_CHECK(lead_char, FLAG_ASCII)) {
-		lead_char = '_';
+	for (i=0;i<mangle_prefix && name[i];i++) {
+		lead_chars[i] = name[i];
+		if (! FLAG_CHECK(lead_chars[i], FLAG_ASCII)) {
+			lead_chars[i] = '_';
+		}
+		lead_chars[i] = toupper(lead_chars[i]);
+	}
+	for (;i<mangle_prefix;i++) {
+		lead_chars[i] = '_';
 	}
-	lead_char = toupper(lead_char);
 
 	/* the prefix is anything up to the first dot */
 	if (dot_p) {
@@ -543,10 +580,12 @@ static void name_map(char *name, BOOL need83, BOOL cache83)
 	v = hash = mangle_hash(name, prefix_len);
 
 	/* now form the mangled name. */
-	new_name[0] = lead_char;
+	for (i=0;i<mangle_prefix;i++) {
+		new_name[i] = lead_chars[i];
+	}
 	new_name[7] = base_forward(v % 36);
 	new_name[6] = '~';	
-	for (i=5; i>=1; i--) {
+	for (i=5; i>=mangle_prefix; i--) {
 		v = v / 36;
 		new_name[i] = base_forward(v % 36);
 	}
@@ -565,7 +604,7 @@ static void name_map(char *name, BOOL need83, BOOL cache83)
 		cache_insert(name, prefix_len, hash);
 	}
 
-	DEBUG(10,("name_map: %s -> %08X -> %s (cache=%d)\n", 
+	M_DEBUG(10,("name_map: %s -> %08X -> %s (cache=%d)\n", 
 		   name, hash, new_name, cache83));
 
 	/* and overwrite the old name */
@@ -588,7 +627,7 @@ static void init_tables(void)
 
 	memset(char_flags, 0, sizeof(char_flags));
 
-	for (i=0;i<128;i++) {
+	for (i=1;i<128;i++) {
 		if ((i >= '0' && i <= '9') || 
 		    (i >= 'a' && i <= 'z') || 
 		    (i >= 'A' && i <= 'Z')) {
@@ -650,6 +689,15 @@ static struct mangle_fns mangle_fns = {
 /* return the methods for this mangling implementation */
 struct mangle_fns *mangle_hash2_init(void)
 {
+	/* the mangle prefix can only be in the mange 1 to 6 */
+	mangle_prefix = lp_mangle_prefix();
+	if (mangle_prefix > 6) {
+		mangle_prefix = 6;
+	}
+	if (mangle_prefix < 1) {
+		mangle_prefix = 1;
+	}
+
 	init_tables();
 	mangle_reset();
 
